//=======================================================================================
// Copyright Nuvoton(SH) Technology Corp. All rights reserved. 2018/8/8   
//=======================================================================================
// 俩定时器Timer0/1的计数频率必须相同且比HCLK低10倍以上,还必须与HCLK是同一时钟源(或选HCLK)
// PWM计数频率必须是HCLK的至少2分频
#ifndef  __Mini5xxDE_BLDC_V16         
#define  __Mini5xxDE_BLDC_V16                      
#include <stdint.h>       
                                    
extern  uint8_t   StartOffset ;       // 起转相偏移,启转函数中写入PhaseOffset决定启转方式
                                      
//函数BLDC_Modify()中用于调整转动过程的变量
extern  uint8_t   PeriodShrink ;      // =24, 赋值32时,起步阶段换相周期不变
extern  uint8_t   Multi_Detect0_Num ; // =1, 过0确认Multi_Detect0_Num+1次
extern  uint16_t  ResonancePoint;     // =158(=0时可直接改High_Speed_Mode)

///////// 起转 /////////////////////////////////////////////////////////////////////////////////////
// 输入: FirstPeriod, 启转周期 
//       DelayMin,    启转阶段退磁时间 ,即换相后到检过0的时间
// 说明: 调用前pPhase,pPhaseEnd,PWM->CMR0 等应先赋值,详见启转代码示例
// 1>, 静止启转：BLDC_Start(PeriodMax,大于1) PeriodMax 是周期上限
// 2>, 低速启转: BLDC_Start(Period,   大于1) Period 是转动周期, 转速较低时启转
// 3>, 直接同步启转: BLDC_Start(Period, 0) 参数二取1表示转速稍低,检过0要多次确认 
// 4>, 回到刚启转状态: BLDC_Start(PeriodMax,0) 参数二可取1 
// 一,BLDC不转时,请用方式1>启转
// 二,顺风起转,测出换相周期做参数1,在换相点按方式2>或3>调用,刚好顺利启转,若转速很低也可按静止起转
// 三,正常转动时,方式4>把状态变到刚启转状态,可加快提速消除顿感. 停转状态时方式4>调用不会启转
void BLDC_Start(uint32_t FirstPeriod, uint32_t DelayMin);     

//////// 转动过程调整,一次换相调用一次, ////////////////////////////////////////////////////////////
// 输入: PhOffset,  取值0,6,12,18...6n。StepCount>=参数二后赋给PhaseOffset(下次过0后的换相才用新值)
//       ActiveNum, StepCount>=此值后, 参数一有效 
// 说明：函数内还包含起转时的加快换相,以及高低速模式切换
// 举例: BLDC_Modify(PhaseOffset, 120);         // 参数一用此变量不改相偏移,可另写代码调整相偏移
void BLDC_Modify(uint8_t PhOffset, uint16_t  ActiveNum) ; 

//////// 测六相电流,此函数可重载 ///////////////////////////////////////////////////////////////////
// 输入: Tim1,           导通时间,最小10*11, 由 PWM 定时,PWM时钟11MHz,实参为100*11就是导通100us
//       *ptrStart,      通电第一相,后6相依次通电
//       CurADC_Channel, 写入寄存器的测电流ADC 通道位, 若是通道5此值就是 0x20
//       *ptr,若指向Cur[0]电流放在0,2,4,6,8,10单元,若指向Cur[1]存放1,3,5,7,9,11,指针为0就把数据丢弃
// 返回: 低16位是电流最大的相,值0~5, 高16位是电流ADC最小值(未减0电流ADC值) 
// 说明: 用到Delayus(),测量时若发生Brake,测量值会不准, ADC 采样率配置不能低于 150Ksps
// 举例: BLDC_Get_Current_Min(80*11, &TabPhase[13], 0x80, (int16_t*)0)      // 导通80us,数据丢弃
int32_t BLDC_Get_Current_Min(int32_t Tim1, uint32_t*ptrStart, uint32_t ADChannel_Cur,int16_t*ptr);

//======================================================================================= 
//////  Control Variable of BLDC //////
extern  uint32_t          *pPhaseEnd ;    // 转动最后一相, ++pPhase大于此减6回到第一相
extern  uint8_t            PhaseOffset ;  // 换相输出是*(pPhase+PhaseOffset),所以决定转动方式
extern   int8_t            PhaseAngle ;   // =22,导通角,过0到换相的延时,30=约30度,前几圈建议取15左右  
extern  uint32_t           First_Delay;   // =3ms,第1次检过0前的通电时间,参考能同步的最慢换相周期  
extern  uint32_t           PeriodMax  ;   // =60ms,周期最大值,参考First_Delay的5~10倍值
extern  uint32_t           DetectUpBit;   // = DETEC_UP=0x80,  转动数组中、检上沿标志位
extern  uint32_t           NxtLngDlyBit;  // = 0x80, All are Long delay if it=0xFFFF
extern  uint32_t           AcmpOutBit ;   // = ACMP_OUT0, 比较器寄存器的输出对应位
extern  uint16_t           FilterRCTime ; // =0, [0,CNR-0x0F],线短且互补时可取值200(0x10,CNR-0x10) 
extern  uint8_t            Num_LongLag ;  // =3
extern  uint8_t        High_Speed_Mode ; // 非0: High speed mode, ResonancePoint=0时可修改
//extern  uint32_t volatile ZeroDeadline; // 执行ZeroDeadline=TIMER1->DR可结束检过0(满足超时条件)                                        
                                          // 执行 ++StepCount;会认为已检到过0
//////  Status Variable of BLDC ////////  
extern  uint32_t *volatile pPhase    ;    // 当前相,检到过0就指向下一相,并把下一相输出值写入PHCHGNXT           
extern  uint32_t  volatile PeriodNow ;    // 当前换相周期,可用于计算转速 
extern  uint32_t  volatile StepCount ;    // 过0计数, 大于9算起转成功
extern  uint32_t  volatile ZeroTick ;     // 上次过0点Timer1的值
//extern uint8_t  volatile Speed_Too_Fast;// 用于测试, Detecting is later for Rotation

// Timer1的检过0延时,总是长于Timer0的换相延时,即：检过0的开始时刻,总是在PHCHGNXT写入PHCHG之后,
// 只在Timer1中断开始判ACMP的哪脚检上沿或下沿, 所以,检过0开始后,就不要更换输出相和检过0相了
#endif   // __Mini5xxDE_BLDC_H 
